<?php

/**
 * @file
 * Additional functions.
 */

/**
 * @return array
 * Define the new image style effect.
 */
function resmushit_image_effect_info() {
  return array(
    'reSmush.it' => array(
      'label' => t('reSmush.it'),
      'help' => t('Optimize images using reSmush.it.'),
      'effect callback' => 'resmushit_effect',
    )
  );
}

/**
 * @param \stdClass $image
 * @return bool
 * Execute image optimalization through reSmush.it
 */
function resmushit_effect(stdClass $image) {
  $debug = FALSE;
  if (variable_get('resmushit_debug', 0) == 1) {
    $debug = TRUE;
  }

  $dst = drupal_realpath($image->source);
  //dpm($dst);
  $apiurl = 'http://api.resmush.it/ws.php'; // See https://www.resmush.it/api

  if ($debug) {
    //dpm('Using drupal_http_request().');
    watchdog('reSmush.it', 'Using drupal_http_request().', WATCHDOG_INFO);
  }
  $boundary = 'A0sFSD';
  $filename = basename($dst);
  //dpm($filename);
  $mimetype = "application/octet-stream";
  $fileencoded = "--$boundary\r\n";
  $fileencoded .= "Content-Disposition: form-data; name=\"files\"; filename=\"$filename\"\r\n";
  $fileencoded .= "Content-Transfer-Encoding: binary\r\n";
  $fileencoded .= "Content-Type: $mimetype\r\n\r\n";
  $fileencoded .= file_get_contents($dst) . "\r\n";
  $fileencoded .="--$boundary--";
  $options = array(
    'headers' => array("Content-Type" => "multipart/form-data; boundary=$boundary"),
    'method' => 'POST',
    'timeout' => variable_get('resmushit_timeout', RESMUSHIT_TIMEOUT),
    'data' => $fileencoded,
  );
  //dpm($options);
  $gotten = drupal_http_request($apiurl, $options);
  //dpm($gotten);

  if ($gotten->code <> 200) {
    // Important to test for an error immediately here. If no response from WP or other error, end immediately.
    watchdog('reSmush.it', 'reSmush.it could not optimize image %dst -- the response code was: %gottencode.', array('%dst' => $image->source, '%gottencode' => $gotten->code), WATCHDOG_ERROR);

    // Keep a log of last few optimalizations.
    $recent_optimalizations = resmushit_trimmed_log();
    $this_operation = array(
      date("Y-m-d H:i:s", REQUEST_TIME),
      $image->source,
      'Error ' . $gotten->code,
      '',
      '',
      '',
    );
    $recent_optimalizations[] = $this_operation;
    variable_set('resmushit_recent_optimalizations', $recent_optimalizations);

    return FALSE;
  }

  $jsondecoded = json_decode($gotten->data);
  //dpm($jsondecoded);

  if (@$jsondecoded->error > 200) {
    // reSmush.it responded but has the image had NOT been optimized?
    //dpm('reSmush.it could not optimize image ' . $image->source . ' (response code was: ... ' . $jsondecoded->error . ' : ' . $jsondecoded->error_long);
    watchdog('reSmush.it', 'reSmush.it could not optimize image %dst -- the response code was: %gottencode1: %gottencode2.', array('%dst' => $image->source, '%gottencode1' => $jsondecoded->error, '%gottencode2' => $jsondecoded->error_long), WATCHDOG_ERROR);

    // Keep a log of last few optimalizations.
    $recent_optimalizations = resmushit_trimmed_log();
    $this_operation = array(
      date("Y-m-d H:i:s", REQUEST_TIME),
      $image->source,
      $jsondecoded->error . ' : ' . $jsondecoded->error_long,
      '',
      '',
      '',
    );
    $recent_optimalizations[] = $this_operation;
    variable_set('resmushit_recent_optimalizations', $recent_optimalizations);

    return FALSE;
  }

  // reSmush.it web service did the job and we can download the optimized image
  $result = drupal_http_request($jsondecoded->dest);
  //dpm($result);
  if (!isset($result->error)) {
    $optimized_image_resource = imagecreatefromstring($result->data);
    $image->resource = $optimized_image_resource;
    if ($debug) {
      //dpm("reSmush.it has successfully optimized image " . $image->source . ". Size reduction " . number_format($jsondecoded->src_size) . " bytes -> " . number_format($jsondecoded->dest_size) . " bytes ($jsondecoded->percent%).");
      watchdog('reSmush.it', 'reSmush.it has successfully optimized image %dst. Size reduction %before bytes -> %after bytes (%percent%).', array('%dst' => $image->source, '%before' => number_format($jsondecoded->src_size), '%after' => number_format($jsondecoded->dest_size), '%percent' => $jsondecoded->percent), WATCHDOG_INFO);
    }

    // Increase the counter of successful optimalizations.
    $total_successes = variable_get('resmushit_total_successes', 0);
    $total_successes++;
    variable_set('resmushit_total_successes', $total_successes);

    // Keep a log of last few optimalizations.
    $recent_optimalizations = resmushit_trimmed_log();
    $this_operation = array(
      date("Y-m-d H:i:s", REQUEST_TIME),
      $image->source,
      'SUCCESS',
      number_format($jsondecoded->src_size) . 'bytes',
      number_format($jsondecoded->dest_size) . 'bytes',
      $jsondecoded->percent . '%',
    );
    $recent_optimalizations[] = $this_operation;
    variable_set('resmushit_recent_optimalizations', $recent_optimalizations);

    return TRUE;
  }
  else {
    // We did not manage to retrieve the optimized image.
    return FALSE;
  }

}

/**
 * Information relevant to the reSmush.it module activity on this site.
 */
function resmushit_dashboard() {
  $result = '';

  $result .= t('<p><h2>reSmush.it module dashboard</h2>');

  $result .= '<p>' . t('' . l('reSmush.it', 'https://www.resmush.it/', array( 'external' => TRUE,'attributes' => array('target' => '_blank') )) . ' is a free image optimalization API that seamlessly replaced the excellent but now defunct Yahoo Smush.it web service. It compresses the size of your images, making them and therefore your website significantly faster to load.');

  $result .= t('<p><h3>Image styles</h3>');
  $result .= '<p>' . t('Effect reSmush.it is currently used by the following image styles on this site:');
  $image_styles = image_styles();
  $configured_image_styles = array();
  foreach($image_styles as $style) {
    foreach($style['effects'] as $arr) {
      if ($arr['label'] == 'reSmush.it') {
        $configured_image_styles[$style['name']] = $style['label'];
        break;
      }
    }
  }
  $list_image_styles = array();
  foreach($configured_image_styles as $key => $val) {
    $list_image_styles[] = l($val, '/admin/config/media/image-styles/edit/' . $key);
  }
  if (empty($list_image_styles)) {
    $list_image_styles[] = 'None yet. Configure them at ' . l('admin/config/media/image-styles', '/admin/config/media/image-styles') . '.';
  }
  $result .= theme_item_list(array(
    'items' => $list_image_styles,
    'title' => '', //'List of image styles using the reSmush.it effect',
    'type' => 'ul',
    'attributes' => array(),
  ));

  $result .= t('<p><h3>Successes</h3>');
  $result .= '<p>' . t('Total number of successful optimalizations on this site: ' . variable_get('resmushit_total_successes', 0)) . '.';

  $result .= t('<p><h3>JPEG image quality</h3>');
  $result .= '<p>' . t('JPEG image quality is set to ' . variable_get('image_jpeg_quality', 75) . '. reSmush.it recommends a value of at least 92 (see ' . l('here', 'https://www.resmush.it/api', array( 'external' => TRUE,'attributes' => array('target' => '_blank') )) . '). That you can change at ' . l('/admin/config/media/image-toolkit', '/admin/config/media/image-toolkit') . '.' );

  $result .= '<p><h3>' . t('Recent %logsize operations: ', array('%logsize' => variable_get('resmushit_log_size', RESMUSHIT_TIMEOUT))) . '</h3>';
  $recent_optimalizations = variable_get('resmushit_recent_optimalizations', array());
  $recent_optimalizations = array_reverse($recent_optimalizations);
  //dpm($recent_optimalizations);
  if (!empty($recent_optimalizations)) {
    $result .= '<table>';
    $result .= '<tr>';
    $result .= '<th>#</th>';
    $result .= '<th>Time</th>';
    $result .= '<th>Image</th>';
    $result .= '<th>Result</th>';
    $result .= '<th>Original size</th>';
    $result .= '<th>Optimized size</th>';
    $result .= '<th>Reduction in size</th>';
    $result .= '</tr>';
    foreach($recent_optimalizations as $keyrow => $row) {
      $result .= '<tr>';
      $result .= '<td>' . ($keyrow+1) . '.</td>';
      foreach($row as $col) {
        $result .= '<td>';
        $result .= $col;
        $result .= '</td>';
      }
      $result .= '</tr>';
    }
    $result .= '</table>';
  }

  return $result;
}

/**
 * @return null
 * Return the log trimmed to the correct length.
 */
function resmushit_trimmed_log() {
  $recent_optimalizations = variable_get('resmushit_recent_optimalizations', array());
  if (sizeof($recent_optimalizations) >= variable_get('resmushit_log_size', RESMUSHIT_LOG_SIZE)) {
    array_shift($recent_optimalizations);
  }
  return $recent_optimalizations;
}
